//#
//# Utility to Get the Name Server IOR File
//# $Revision: 1.1 $
//# Copyright 2004 by Eric Y. Theriault
//# All Rights Reserved.
//# http://www.eyt.ca/CORBA
//#
package ca.eyt.corba;

/**
 * This utility will acquire the Name Server's IOR file.
 *
 * @author Eric Y. Theriault <eric@eyt.ca>
 */
public class NameServerIOR {
   /**
    * main
    *
    * @param args Currently forwarded directly to ORB.init.
    */
   public static void main( String args[] ) {
      try {
         String iorFile = "nameserver.ior";

         // Create and Initialize the ORB
         org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init( args, null );

         // Let's get the root naming context.
         org.omg.CORBA.Object objectReference =
                         orb.resolve_initial_references( "NameService" );
         org.omg.CosNaming.NamingContextExt ncRef =
                       org.omg.CosNaming.NamingContextExtHelper.narrow(
                                                         objectReference
                                                                      );

         // Create the file.
         java.io.FileWriter fileWriter = null;
         java.io.BufferedWriter writer = null;

         // Write a file with the IOR
         String ior = orb.object_to_string( ncRef );
         try {
            fileWriter = new java.io.FileWriter( iorFile );
            writer = new java.io.BufferedWriter( fileWriter );
         } catch ( java.io.IOException ioe ) {
            System.err.println( iorFile + ": Failure to create:" );
            ioe.printStackTrace();
            System.exit( 1 );
         }

         // Write in the IOR from the file...
         try {
            writer.write( ior );
            writer.newLine();
         } catch ( java.io.IOException ioe ) {
            System.err.println( iorFile + ": exception raised: " );
            ioe.printStackTrace();
            System.exit( 1 );
         } finally {
            writer.flush();
            try {
               fileWriter.close();
            } catch ( java.io.IOException ioe ) {
               System.err.println( iorFile + ": exception raised " +
                                   "closing the file: " );
               ioe.printStackTrace();
            }
            writer = null;
            fileWriter = null;
         }
      } catch ( Throwable t ) {
         System.err.println( "Exception raised: " );
         t.printStackTrace();
         System.exit( 1 );
      }
      System.exit( 0 );
   }
};
